// Includes ////////////////////////////////////////////////////////////////

#include "MainWindow.h"

// Constants ///////////////////////////////////////////////////////////////

const std::wstring MainWindow::Name = L"MainWindow";
const std::wstring MainWindow::Title = L"Scaled Resolution Editor Beta 4 by ToastyX";

const int MainWindow::ClientWidth = 400;
const int MainWindow::ClientHeight = 640;

const std::wstring MainWindow::DefaultExtension = L"txt";

const COMDLG_FILTERSPEC MainWindow::FileTypes[] =
{
	{ L"Text Files", L"*.txt" },
	{ L"All Files", L"*.*" }
};

// Functions ///////////////////////////////////////////////////////////////

MainWindow::MainWindow(const SRE::GpuList &gpuList) :
	Window(Name, Title, WS_OVERLAPPEDWINDOW),
	gpuLabel(*this, L"GPU:"),
	gpuComboBox(*this),
	resolutionsGroupBox(*this, L" Resolutions "),
	widthTextBox(*this),
	xLabel(*this, L"\xD7"),
	heightTextBox(*this),
	addButton(*this, L"+"),
	deleteButton(*this, L"\x2013"),
	deleteAllButton(*this, L"Delete all"),
	defaultButton(*this, L"Default"),
	resolutionsListBox(*this),
	importButton(*this, L"Import..."),
	exportButton(*this, L"Export..."),
	copyButton(*this, L"Copy"),
	pasteButton(*this, L"Paste"),
	okButton(*this, L"OK"),
	cancelButton(*this, L"Cancel"),
	gpuList(gpuList),
	gpuIndex(-1),
	copied(false)
{
	AddMessages();
	AddCommands();
	InitializeTheme();
	InitializeControls();
	ScaleControls(true);
	Center();
	Show();
}

////////////////////////////////////////////////////////////////////////////

LRESULT MainWindow::OnSize(WPARAM wParam, LPARAM lParam)
{
	ScaleControls();
	return 0;
}

////////////////////////////////////////////////////////////////////////////

LRESULT MainWindow::OnClose(WPARAM wParam, LPARAM lParam)
{
	UpdateGpuList();

	if (gpuList.IsSaved())
	{
		CancelButtonClick();
		return 0;
	}

	UI::MessageBox messageBox(*this, L"Do you want to save changes?", L"Close", L"Save", L"Don't Save", L"Cancel");

	switch (messageBox.MessageLoop())
	{
		case 1:
			OkButtonClick();
			return 0;

		case 2:
			CancelButtonClick();
			return 0;
	}

	return 0;
}

////////////////////////////////////////////////////////////////////////////

LRESULT MainWindow::OnSysColorChange(WPARAM wParam, LPARAM lParam)
{
	InitializeTheme();
	return 0;
}

////////////////////////////////////////////////////////////////////////////

LRESULT MainWindow::OnSettingChange(WPARAM wParam, LPARAM lParam)
{
	if (lParam)
	{
		std::wstring setting = reinterpret_cast<wchar_t *>(lParam);

		if (setting == L"ImmersiveColorSet")
			InitializeTheme();
	}

	return 0;
}

////////////////////////////////////////////////////////////////////////////

LRESULT MainWindow::OnDpiChanged(WPARAM wParam, LPARAM lParam)
{
	InitializeTheme();
	ScaleControls();
	RECT *rect = reinterpret_cast<RECT *>(lParam);
	SetPosition(rect->left, rect->top);
	SetRightBottom(rect->right, rect->bottom);
	return 0;
}

////////////////////////////////////////////////////////////////////////////

LRESULT MainWindow::OnThemeChanged(WPARAM wParam, LPARAM lParam)
{
	InitializeTheme();
	return 0;
}

////////////////////////////////////////////////////////////////////////////

LRESULT MainWindow::OnClipboardUpdate(WPARAM wParam, LPARAM lParam)
{
	UpdateCopyText();
	RefreshCopyPasteButtons();
	return 0;
}

////////////////////////////////////////////////////////////////////////////

bool MainWindow::AddMessages()
{
	using namespace std::placeholders;
	AddMessage(WM_SIZE, std::bind(&MainWindow::OnSize, this, _1, _2));
	AddMessage(WM_CLOSE, std::bind(&MainWindow::OnClose, this, _1, _2));
	AddMessage(WM_SYSCOLORCHANGE, std::bind(&MainWindow::OnSysColorChange, this, _1, _2));
	AddMessage(WM_SETTINGCHANGE, std::bind(&MainWindow::OnSettingChange, this, _1, _2));
	AddMessage(WM_DPICHANGED, std::bind(&MainWindow::OnDpiChanged, this, _1, _2));
	AddMessage(WM_THEMECHANGED, std::bind(&MainWindow::OnThemeChanged, this, _1, _2));
	AddMessage(WM_CLIPBOARDUPDATE, std::bind(&MainWindow::OnClipboardUpdate, this, _1, _2));
	AddClipboardFormatListener(GetHandle());
	return true;
}

////////////////////////////////////////////////////////////////////////////

bool MainWindow::RefreshGpuComboBox()
{
	SRE::Gpu gpu;
	gpuComboBox.BeginUpdate();
	gpuComboBox.Clear();

	for (int index = 0; gpuList.Get(index, gpu); ++index)
	{
		std::wstring text = gpu.GetName();

		if (gpu.IsActive())
			text += L" (active)";

		if (gpu.IsModified())
			text += L"*";

		gpuComboBox.Add(text);
	}

	RefreshGpuComboBoxColors();
	gpuComboBox.EndUpdate();
	return true;
}

////////////////////////////////////////////////////////////////////////////

bool MainWindow::RefreshGpuComboBoxColors()
{
	SRE::Gpu gpu;
	UI::Theme theme = GetTheme();

	for (int index = 0; gpuList.Get(index, gpu); ++index)
	{
		if (gpu.IsActive())
			gpuComboBox.SetTextColors(index, theme.ComboBoxListTextColors);
		else
			gpuComboBox.SetTextColors(index, theme.ComboBoxListGrayTextColors);
	}

	return true;
}

////////////////////////////////////////////////////////////////////////////

bool MainWindow::RefreshResolutionTextBoxes()
{
	SRE::Resolution resolution;

	if (!resolutionList.Get(resolutionsListBox.GetIndex(), resolution))
	{
		widthTextBox.Clear();
		heightTextBox.Clear();
	}
	else
	{
		widthTextBox.SetInteger(resolution.width);
		heightTextBox.SetInteger(resolution.height);
	}

	RefreshResolutionTextBoxesColors();
	return true;
}

////////////////////////////////////////////////////////////////////////////

bool MainWindow::RefreshResolutionTextBoxesColors()
{
	int value;
	UI::Theme theme = GetTheme();

	if (widthTextBox.GetInteger(value) && resolutionList.IsValidWidth(value))
		widthTextBox.SetTextColors(theme.TextBoxTextColors);
	else
		widthTextBox.SetTextColors(theme.TextBoxInvalidTextColors);

	if (heightTextBox.GetInteger(value) && resolutionList.IsValidHeight(value))
		heightTextBox.SetTextColors(theme.TextBoxTextColors);
	else
		heightTextBox.SetTextColors(theme.TextBoxInvalidTextColors);

	return true;
}

////////////////////////////////////////////////////////////////////////////

bool MainWindow::RefreshAddButton()
{
	SRE::Resolution resolution;

	if (!widthTextBox.GetInteger(resolution.width) || !resolutionList.IsValidWidth(resolution.width))
	{
		addButton.Disable();
		return true;
	}

	if (!heightTextBox.GetInteger(resolution.height) || !resolutionList.IsValidHeight(resolution.height))
	{
		addButton.Disable();
		return true;
	}

	if (resolutionList.IsAdded(resolution))
	{
		addButton.Disable();
		return true;
	}

	addButton.Enable();
	return true;
}

////////////////////////////////////////////////////////////////////////////

bool MainWindow::RefreshDeleteButton()
{
	if (resolutionsListBox.GetIndex() < 0)
		deleteButton.Disable();
	else
		deleteButton.Enable();

	return true;
}

////////////////////////////////////////////////////////////////////////////

bool MainWindow::RefreshDeleteAllButton()
{
	if (resolutionList.GetCount() <= 0)
		deleteAllButton.Disable();
	else
		deleteAllButton.Enable();

	return true;
}

////////////////////////////////////////////////////////////////////////////

bool MainWindow::RefreshDefaultButton()
{
	if (resolutionList == gpu.GetDefaultResolutionList())
		defaultButton.Disable();
	else
		defaultButton.Enable();

	return true;
}

////////////////////////////////////////////////////////////////////////////

bool MainWindow::RefreshResolutionsListBox(int top)
{
	SRE::Resolution resolution;
	resolutionsListBox.BeginUpdate();
	resolutionsListBox.Clear();

	for (int index = 0; resolutionList.Get(index, resolution); ++index)
		resolutionsListBox.Add(resolution.GetText());

	if (top > 0)
		resolutionsListBox.SetTopIndex(top);

	RefreshResolutionsListBoxColors();
	resolutionsListBox.EndUpdate();
	return true;
}

////////////////////////////////////////////////////////////////////////////

bool MainWindow::RefreshResolutionsListBoxColors()
{
	SRE::Resolution resolution;
	UI::Theme theme = GetTheme();

	for (int index = 0; resolutionList.Get(index, resolution); ++index)
	{
		if (resolution.added)
			resolutionsListBox.SetTextColors(index, theme.ListBoxTextColors);
		else
			resolutionsListBox.SetTextColors(index, theme.ListBoxGrayTextColors);
	}

	return true;
}

////////////////////////////////////////////////////////////////////////////

bool MainWindow::RefreshResolutionsListBoxIndex()
{
	SRE::Resolution resolution;

	if (!widthTextBox.GetInteger(resolution.width) || !heightTextBox.GetInteger(resolution.height))
		resolutionsListBox.Deselect();
	else
		resolutionsListBox.SetIndex(resolutionList.GetIndex(resolution));

	return true;
}

////////////////////////////////////////////////////////////////////////////

bool MainWindow::RefreshCopyPasteButtons()
{
	if (!copied)
	{
		copyButton.Enable();
		pasteButton.Disable();
		return true;
	}

	if (resolutionList.GetText() == copiedText)
		copyButton.Disable();
	else
		copyButton.Enable();

	SRE::ResolutionList pasteResolutionList = resolutionList;
	pasteResolutionList.SetText(copiedText);

	if (resolutionList == pasteResolutionList)
		pasteButton.Disable();
	else
		pasteButton.Enable();

	return true;
}

////////////////////////////////////////////////////////////////////////////

void MainWindow::UpdateGpuList()
{
	if (!gpuList.IsValidIndex(gpuIndex))
		return;

	if (!gpu.SetResolutionList(resolutionList))
		return;

	gpuList.Set(gpuIndex, gpu);
}

////////////////////////////////////////////////////////////////////////////

void MainWindow::UpdateCopyText()
{
	UI::Clipboard clipboard(*this);
	copied = clipboard.GetText(copiedText);
}

////////////////////////////////////////////////////////////////////////////

void MainWindow::GpuComboBoxChange()
{
	UpdateGpuList();
	gpuIndex = gpuComboBox.GetIndex();

	if (!gpuList.Get(gpuIndex, gpu))
		return;

	resolutionList = gpu.GetResolutionList();
	RefreshResolutionsListBox();
	RefreshResolutionTextBoxes();
	RefreshAddButton();
	RefreshDeleteButton();
	RefreshDeleteAllButton();
	RefreshDefaultButton();
	RefreshCopyPasteButtons();
}

////////////////////////////////////////////////////////////////////////////

void MainWindow::WidthTextBoxEnter()
{
	addButton.SetDefault(true);
}

////////////////////////////////////////////////////////////////////////////

void MainWindow::WidthTextBoxChange()
{
	RefreshResolutionTextBoxesColors();
	RefreshResolutionsListBoxIndex();
	RefreshAddButton();
	RefreshDeleteButton();
	addButton.SetDefault(true);
}

////////////////////////////////////////////////////////////////////////////

void MainWindow::WidthTextBoxExit()
{
	int width;

	if (!widthTextBox.GetInteger(width) || !resolutionList.IsValidWidth(width))
		return;

	widthTextBox.SetInteger(width);
	addButton.SetDefault(false);
}

////////////////////////////////////////////////////////////////////////////

void MainWindow::HeightTextBoxEnter()
{
	addButton.SetDefault(true);
}

////////////////////////////////////////////////////////////////////////////

void MainWindow::HeightTextBoxChange()
{
	RefreshResolutionTextBoxesColors();
	RefreshResolutionsListBoxIndex();
	RefreshAddButton();
	RefreshDeleteButton();
	addButton.SetDefault(true);
}

////////////////////////////////////////////////////////////////////////////

void MainWindow::HeightTextBoxExit()
{
	int height;

	if (!heightTextBox.GetInteger(height) || !resolutionList.IsValidHeight(height))
		return;

	heightTextBox.SetInteger(height);
	addButton.SetDefault(false);
}

////////////////////////////////////////////////////////////////////////////

void MainWindow::AddButtonClick()
{
	SRE::Resolution resolution;

	if (!widthTextBox.GetInteger(resolution.width))
		return;

	if (!heightTextBox.GetInteger(resolution.height))
		return;

	if (!resolutionList.Add(resolution))
		return;

	RefreshResolutionsListBox(resolutionsListBox.GetTopIndex());
	RefreshResolutionsListBoxIndex();
	RefreshAddButton();
	RefreshDeleteButton();
	RefreshDeleteAllButton();
	RefreshDefaultButton();
	RefreshCopyPasteButtons();
	widthTextBox.Select();
	widthTextBox.SetFocus();
}

////////////////////////////////////////////////////////////////////////////

void MainWindow::DeleteButtonClick()
{
	int index = resolutionsListBox.GetIndex();

	if (!resolutionList.Delete(index))
		return;

	RefreshResolutionsListBox(resolutionsListBox.GetTopIndex());
	resolutionsListBox.SetIndex(index);
	RefreshResolutionTextBoxes();
	RefreshAddButton();
	RefreshDeleteButton();
	RefreshDeleteAllButton();
	RefreshDefaultButton();
	RefreshCopyPasteButtons();

	if (!deleteButton.IsEnabled())
		widthTextBox.SetFocus();
}

////////////////////////////////////////////////////////////////////////////

void MainWindow::DeleteAllButtonClick()
{
	if (!resolutionList.Clear())
		return;

	RefreshResolutionsListBox();
	RefreshResolutionTextBoxes();
	RefreshAddButton();
	RefreshDeleteButton();
	RefreshDeleteAllButton();
	RefreshDefaultButton();
	RefreshCopyPasteButtons();
	widthTextBox.SetFocus();
}

////////////////////////////////////////////////////////////////////////////

void MainWindow::DefaultButtonClick()
{
	resolutionList = gpu.GetDefaultResolutionList();
	RefreshResolutionsListBox();
	RefreshResolutionTextBoxes();
	RefreshAddButton();
	RefreshDeleteButton();
	RefreshDeleteAllButton();
	RefreshDefaultButton();
	RefreshCopyPasteButtons();
	widthTextBox.SetFocus();
}

////////////////////////////////////////////////////////////////////////////

void MainWindow::ResolutionsListBoxChange()
{
	RefreshResolutionTextBoxes();
	RefreshAddButton();
	RefreshDeleteButton();
}

////////////////////////////////////////////////////////////////////////////

void MainWindow::ImportButtonClick()
{
	bool initialized = false;
	IFileOpenDialog *openDialog = nullptr;
	IShellItem *result = nullptr;
	wchar_t *file = nullptr;

	try
	{
		if (!SUCCEEDED(CoInitializeEx(nullptr, COINIT_APARTMENTTHREADED | COINIT_DISABLE_OLE1DDE)))
			throw L"CoInitializeEx failed.";

		initialized = true;

		if (!SUCCEEDED(CoCreateInstance(CLSID_FileOpenDialog, nullptr, CLSCTX_INPROC_SERVER, IID_PPV_ARGS(&openDialog))))
			throw L"CoCreateInstance failed.";

		if (!SUCCEEDED(openDialog->SetTitle(L"Import")))
			throw L"IFileSaveDialog::SetTitle failed.";

		if (!SUCCEEDED(openDialog->SetOkButtonLabel(L"Import")))
			throw L"IFileSaveDialog::SetOkButtonLabel failed.";

		if (!SUCCEEDED(openDialog->SetDefaultExtension(DefaultExtension.c_str())))
			throw L"IFileSaveDialog::SetDefaultExtension failed.";

		if (!SUCCEEDED(openDialog->SetFileTypes(ARRAYSIZE(FileTypes), FileTypes)))
			throw L"IFileSaveDialog::SetFileTypes failed.";

		if (SUCCEEDED(openDialog->Show(GetHandle())))
		{
			if (!SUCCEEDED(openDialog->GetResult(&result)))
				throw L"IFileSaveDialog::GetResult failed.";

			if (!SUCCEEDED(result->GetDisplayName(SIGDN_FILESYSPATH, &file)))
				throw L"IShellItem::GetDisplayName failed.";

			if (!resolutionList.Import(file))
				throw L"Failed to import file.";

			RefreshResolutionsListBox();
			RefreshResolutionTextBoxes();
			RefreshAddButton();
			RefreshDeleteButton();
			RefreshDeleteAllButton();
			RefreshDefaultButton();
			RefreshCopyPasteButtons();
		}
	}
	catch (const wchar_t *message)
	{
		MessageBoxW(GetHandle(), message, L"Error", MB_ICONERROR);
	}

	if (file)
		CoTaskMemFree(file);

	if (result)
		result->Release();

	if (openDialog)
		openDialog->Release();

	if (initialized)
		CoUninitialize();
}

////////////////////////////////////////////////////////////////////////////

void MainWindow::ExportButtonClick()
{
	bool initialized = false;
	IFileSaveDialog *saveDialog = nullptr;
	IShellItem *result = nullptr;
	wchar_t *file = nullptr;

	try
	{
		if (!SUCCEEDED(CoInitializeEx(nullptr, COINIT_APARTMENTTHREADED | COINIT_DISABLE_OLE1DDE)))
			throw L"CoInitializeEx failed.";

		initialized = true;

		if (!SUCCEEDED(CoCreateInstance(CLSID_FileSaveDialog, nullptr, CLSCTX_INPROC_SERVER, IID_PPV_ARGS(&saveDialog))))
			throw L"CoCreateInstance failed.";

		if (!SUCCEEDED(saveDialog->SetTitle(L"Export")))
			throw L"IFileSaveDialog::SetTitle failed.";

		if (!SUCCEEDED(saveDialog->SetOkButtonLabel(L"Export")))
			throw L"IFileSaveDialog::SetOkButtonLabel failed.";

		if (!SUCCEEDED(saveDialog->SetDefaultExtension(DefaultExtension.c_str())))
			throw L"IFileSaveDialog::SetDefaultExtension failed.";

		if (!SUCCEEDED(saveDialog->SetFileTypes(ARRAYSIZE(FileTypes), FileTypes)))
			throw L"IFileSaveDialog::SetFileTypes failed.";

		if (SUCCEEDED(saveDialog->Show(GetHandle())))
		{
			if (!SUCCEEDED(saveDialog->GetResult(&result)))
				throw L"IFileSaveDialog::GetResult failed.";

			if (!SUCCEEDED(result->GetDisplayName(SIGDN_FILESYSPATH, &file)))
				throw L"IShellItem::GetDisplayName failed.";

			if (!resolutionList.Export(file))
				throw L"Failed to export file.";
		}
	}
	catch (const wchar_t *message)
	{
		MessageBoxW(GetHandle(), message, L"Error", MB_ICONERROR);
	}

	if (file)
		CoTaskMemFree(file);

	if (result)
		result->Release();

	if (saveDialog)
		saveDialog->Release();

	if (initialized)
		CoUninitialize();
}

////////////////////////////////////////////////////////////////////////////

void MainWindow::CopyButtonClick()
{
	UI::Clipboard clipboard(*this);
	std::wstring text = resolutionList.GetText();

	if (!clipboard.SetText(text))
		return;

	copiedText = text;
	copied = true;
	RefreshCopyPasteButtons();
}

////////////////////////////////////////////////////////////////////////////

void MainWindow::PasteButtonClick()
{
	UI::Clipboard clipboard(*this);
	std::wstring text;

	if (!clipboard.GetText(text))
		return;

	if (!resolutionList.SetText(text))
		return;

	RefreshResolutionsListBox();
	RefreshResolutionTextBoxes();
	RefreshAddButton();
	RefreshDeleteButton();
	RefreshDeleteAllButton();
	RefreshDefaultButton();
	RefreshCopyPasteButtons();
}

////////////////////////////////////////////////////////////////////////////

void MainWindow::OkButtonClick()
{
	UpdateGpuList();

	int count = gpuList.Save();

	if (count <= 0)
		throw L"Failed to save changes.";

	if (count != gpuList.GetCount())
		throw L"Failed to save some changes.";

	Close(0);
}

////////////////////////////////////////////////////////////////////////////

void MainWindow::CancelButtonClick()
{
	Close(0);
}

////////////////////////////////////////////////////////////////////////////

void MainWindow::EnterKeyPress()
{
	if (!widthTextBox.IsFocused() && !heightTextBox.IsFocused())
		return;

	AddButtonClick();
}

////////////////////////////////////////////////////////////////////////////

void MainWindow::EscKeyPress()
{
	CancelButtonClick();
}

////////////////////////////////////////////////////////////////////////////

bool MainWindow::AddCommands()
{
	gpuComboBox.AddCommand(CBN_SELCHANGE, std::bind(&MainWindow::GpuComboBoxChange, this));
	widthTextBox.AddCommand(EN_SETFOCUS, std::bind(&MainWindow::WidthTextBoxEnter, this));
	widthTextBox.AddCommand(EN_CHANGE, std::bind(&MainWindow::WidthTextBoxChange, this));
	widthTextBox.AddCommand(EN_KILLFOCUS, std::bind(&MainWindow::WidthTextBoxExit, this));
	heightTextBox.AddCommand(EN_SETFOCUS, std::bind(&MainWindow::HeightTextBoxEnter, this));
	heightTextBox.AddCommand(EN_CHANGE, std::bind(&MainWindow::HeightTextBoxChange, this));
	heightTextBox.AddCommand(EN_KILLFOCUS, std::bind(&MainWindow::HeightTextBoxExit, this));
	addButton.AddCommand(BN_CLICKED, std::bind(&MainWindow::AddButtonClick, this));
	deleteButton.AddCommand(BN_CLICKED, std::bind(&MainWindow::DeleteButtonClick, this));
	deleteAllButton.AddCommand(BN_CLICKED, std::bind(&MainWindow::DeleteAllButtonClick, this));
	defaultButton.AddCommand(BN_CLICKED, std::bind(&MainWindow::DefaultButtonClick, this));
	resolutionsListBox.AddCommand(LBN_SELCHANGE, std::bind(&MainWindow::ResolutionsListBoxChange, this));
	importButton.AddCommand(BN_CLICKED, std::bind(&MainWindow::ImportButtonClick, this));
	exportButton.AddCommand(BN_CLICKED, std::bind(&MainWindow::ExportButtonClick, this));
	copyButton.AddCommand(BN_CLICKED, std::bind(&MainWindow::CopyButtonClick, this));
	pasteButton.AddCommand(BN_CLICKED, std::bind(&MainWindow::PasteButtonClick, this));
	okButton.AddCommand(BN_CLICKED, std::bind(&MainWindow::OkButtonClick, this));
	cancelButton.AddCommand(BN_CLICKED, std::bind(&MainWindow::CancelButtonClick, this));
	AddCommand(IDOK, std::bind(&MainWindow::EnterKeyPress, this));
	AddCommand(IDCANCEL, std::bind(&MainWindow::EscKeyPress, this));
	return true;
}

////////////////////////////////////////////////////////////////////////////

bool MainWindow::InitializeTheme()
{
	UI::Dpi dpi = GetDpi();
	UI::SystemTheme systemTheme;
	UI::Theme theme;

	if (systemTheme.IsHighContrast())
		theme = UI::HighContrastTheme(dpi);
	else if (systemTheme.IsDarkMode())
		theme = UI::DarkTheme(dpi);
	else
		theme = UI::LightTheme(dpi);

	SetTheme(theme);
	gpuLabel.SetTheme(theme);
	gpuComboBox.SetTheme(theme);
	resolutionsGroupBox.SetTheme(theme);
	widthTextBox.SetTheme(theme);
	xLabel.SetTheme(theme);
	heightTextBox.SetTheme(theme);
	addButton.SetTheme(theme);
	deleteButton.SetTheme(theme);
	deleteAllButton.SetTheme(theme);
	defaultButton.SetTheme(theme);
	resolutionsListBox.SetTheme(theme);
	importButton.SetTheme(theme);
	exportButton.SetTheme(theme);
	copyButton.SetTheme(theme);
	pasteButton.SetTheme(theme);
	okButton.SetTheme(theme);
	cancelButton.SetTheme(theme);

	RefreshGpuComboBoxColors();
	RefreshResolutionTextBoxesColors();
	RefreshResolutionsListBoxColors();
	return true;
}

////////////////////////////////////////////////////////////////////////////

bool MainWindow::InitializeControls()
{
	RefreshGpuComboBox();
	widthTextBox.SetMaxLength(5);
	heightTextBox.SetMaxLength(5);
	resolutionsListBox.SetText(L"No resolutions");
	UpdateCopyText();
	gpuComboBox.SetIndex(0);
	GpuComboBoxChange();
	widthTextBox.SetFocus();
	return true;
}

////////////////////////////////////////////////////////////////////////////

bool MainWindow::ScaleControls(bool resize)
{
	UI::Dpi dpi = GetDpi();
	UI::Theme theme = GetTheme();
	UI::Font font(theme.FontName, theme.FontSize);

	const int ResolutionTextBoxWidth = theme.Padding + font.GetWidth(L"00000") + theme.Padding;
	const int ResolutionsListBoxMinRows = 5;
	const int ResolutionsListBoxMinHeight = theme.ListBoxPadding + font.GetHeight() * ResolutionsListBoxMinRows + theme.ListBoxPadding;

	SetIcon(L"ICON", 192);

	gpuLabel.SetFont(font);
	gpuLabel.SetPosition(theme.Spacing, theme.Spacing);
	gpuLabel.SetSize(font.GetWidth(gpuLabel.GetText()), theme.LineHeight);

	gpuComboBox.SetFont(font);
	gpuComboBox.SetPosition(gpuLabel.GetRight() + theme.LabelSpacing, gpuLabel.GetY());
	gpuComboBox.SetRightBottom(GetClientWidth() - theme.Spacing, gpuComboBox.GetY() + theme.LineHeight, GetClientHeight() - theme.Spacing);

	resolutionsGroupBox.SetFont(font);
	resolutionsGroupBox.SetPosition(gpuLabel.GetX(), gpuComboBox.GetY() + theme.LineHeight + theme.Spacing + theme.Spacing);
	resolutionsGroupBox.SetRightBottom(GetClientWidth() - theme.Spacing, GetClientHeight() - theme.Spacing - theme.ButtonHeight - theme.Spacing);

	widthTextBox.SetFont(font);
	widthTextBox.SetPosition(resolutionsGroupBox.GetX() + theme.GroupBoxBorderSizes.BorderSize + theme.Spacing, resolutionsGroupBox.GetY() + theme.GroupBoxBorderSizes.BorderSize + theme.Spacing + theme.Spacing);
	widthTextBox.SetSize(ResolutionTextBoxWidth, theme.LineHeight);

	xLabel.SetFont(font);
	xLabel.SetPosition(widthTextBox.GetRight() + theme.LabelSpacing, widthTextBox.GetY());
	xLabel.SetSize(font.GetWidth(xLabel.GetText()), theme.LineHeight);

	heightTextBox.SetFont(font);
	heightTextBox.SetPosition(xLabel.GetRight() + theme.LabelSpacing, xLabel.GetY());
	heightTextBox.SetSize(ResolutionTextBoxWidth, theme.LineHeight);

	addButton.SetFont(font);
	addButton.SetPosition(heightTextBox.GetRight() + theme.LineSpacing, heightTextBox.GetY());
	addButton.SetSize(theme.LineHeight, theme.LineHeight);

	deleteButton.SetFont(font);
	deleteButton.SetPosition(addButton.GetRight() + theme.LineSpacing, addButton.GetY());
	deleteButton.SetSize(theme.LineHeight, theme.LineHeight);

	defaultButton.SetFont(font);
	defaultButton.SetSize(theme.ButtonPadding + font.GetWidth(defaultButton.GetText()) + theme.ButtonPadding, theme.LineHeight);
	defaultButton.SetPosition(resolutionsGroupBox.GetRight() - theme.GroupBoxBorderSizes.BorderSize - theme.Spacing - defaultButton.GetWidth(), deleteButton.GetY());

	deleteAllButton.SetFont(font);
	deleteAllButton.SetSize(theme.ButtonPadding + font.GetWidth(deleteAllButton.GetText()) + theme.ButtonPadding, theme.LineHeight);
	deleteAllButton.SetPosition(defaultButton.GetX() - theme.LineSpacing - deleteAllButton.GetWidth(), defaultButton.GetY());

	resolutionsListBox.SetFont(font);
	resolutionsListBox.SetPosition(widthTextBox.GetX(), widthTextBox.GetBottom() + theme.Spacing);
	resolutionsListBox.SetRightBottom(resolutionsGroupBox.GetRight() - theme.GroupBoxBorderSizes.BorderSize - theme.Spacing, resolutionsGroupBox.GetBottom() - theme.GroupBoxBorderSizes.BorderSize - theme.Spacing - theme.LineHeight - theme.Spacing);

	importButton.SetFont(font);
	importButton.SetPosition(resolutionsListBox.GetX(), resolutionsListBox.GetBottom() + theme.Spacing);
	importButton.SetSize(theme.ButtonPadding + font.GetWidth(importButton.GetText()) + theme.ButtonPadding, theme.LineHeight);

	exportButton.SetFont(font);
	exportButton.SetPosition(importButton.GetRight() + theme.LineSpacing, importButton.GetY());
	exportButton.SetSize(importButton.GetWidth(), theme.LineHeight);

	pasteButton.SetFont(font);
	pasteButton.SetSize(theme.ButtonPadding + font.GetWidth(pasteButton.GetText()) + theme.ButtonPadding, theme.LineHeight);
	pasteButton.SetPosition(resolutionsGroupBox.GetRight() - theme.GroupBoxBorderSizes.BorderSize - theme.Spacing - pasteButton.GetWidth(), exportButton.GetY());

	copyButton.SetFont(font);
	copyButton.SetSize(pasteButton.GetWidth(), theme.LineHeight);
	copyButton.SetPosition(pasteButton.GetX() - theme.LineSpacing - copyButton.GetWidth(), pasteButton.GetY());

	cancelButton.SetFont(font);
	cancelButton.SetPosition(resolutionsGroupBox.GetRight() - theme.ButtonWidth, resolutionsGroupBox.GetBottom() + theme.Spacing);
	cancelButton.SetSize(theme.ButtonWidth, theme.ButtonHeight);

	okButton.SetFont(font);
	okButton.SetPosition(cancelButton.GetX() - theme.Spacing - theme.ButtonWidth, cancelButton.GetY());
	okButton.SetSize(theme.ButtonWidth, theme.ButtonHeight);

	int minWidth = deleteButton.GetRight() + theme.LineSpacing + deleteAllButton.GetWidth() + theme.LineSpacing + defaultButton.GetWidth() + theme.Spacing + theme.GroupBoxBorderSizes.BorderSize + theme.Spacing;
	int minHeight = resolutionsListBox.GetY() + ResolutionsListBoxMinHeight + theme.Spacing + theme.LineHeight + theme.Spacing + theme.GroupBoxBorderSizes.BorderSize + theme.Spacing + theme.ButtonHeight + theme.Spacing;
	SetMinClientSize(minWidth, minHeight);

	if (resize)
	{
		SetClientSize(dpi.Scale(ClientWidth), dpi.Scale(ClientHeight));
		Fit();
	}

	return true;
}

////////////////////////////////////////////////////////////////////////////
