// Includes ////////////////////////////////////////////////////////////////

#include "SRE.h"

// Constants ///////////////////////////////////////////////////////////////

const int SRE::ResolutionList::MinWidth = 100;
const int SRE::ResolutionList::MaxWidth = 15999;
const int SRE::ResolutionList::MinHeight = 100;
const int SRE::ResolutionList::MaxHeight = 15999;

const std::vector<SRE::Resolution> SRE::ResolutionList::AmdDefaultResolutionList =
{
	{5120, 2880, false},
	{5120, 2160, false},
	{3840, 2400, false},
	{3840, 2160, false},
	{3840, 1620, false},
	{3200, 1800, false},
	{3072, 1728, false},
	{2560, 1600, false},
	{2560, 1440, false},
	{2048, 1536, true},
	{1920, 1200, true},
	{1920, 1080, true},
	{1760, 990, false},
	{1680, 1050, true},
	{1600, 1200, true},
	{1600, 1000, false},
	{1600, 900, false},
	{1366, 768, false},
	{1280, 1024, true},
	{1280, 800, false},
	{1280, 720, true},
	{1128, 634, false},
	{1024, 768, true},
	{800, 600, true},
	{640, 480, true},
};

const std::vector<SRE::Resolution> SRE::ResolutionList::NvidiaDefaultResolutionList =
{
	{1600, 1200, true},
	{1280, 1024, true},
	{1024, 768, true},
	{800, 600, true},
	{640, 480, true},
};

// Functions ///////////////////////////////////////////////////////////////

SRE::ResolutionList::ResolutionList() :
	type(GpuType::Invalid)
{
}

////////////////////////////////////////////////////////////////////////////

bool SRE::ResolutionList::operator==(const ResolutionList &other) const
{
	if (resolutionList.size() != other.resolutionList.size())
		return false;

	auto iterator1 = resolutionList.begin();
	auto iterator2 = other.resolutionList.begin();

	while (iterator1 != resolutionList.end())
	{
		if (*iterator1 != *iterator2 || iterator1->added != iterator2->added)
			return false;

		++iterator1;
		++iterator2;
	}

	return true;
}

////////////////////////////////////////////////////////////////////////////

bool SRE::ResolutionList::operator!=(const ResolutionList &other) const
{
	return !(*this == other);
}

////////////////////////////////////////////////////////////////////////////

int SRE::ResolutionList::GetCount() const
{
	return static_cast<int>(resolutionList.size());
}

////////////////////////////////////////////////////////////////////////////

bool SRE::ResolutionList::IsValidIndex(int index) const
{
	return index >= 0 && index < GetCount();
}

////////////////////////////////////////////////////////////////////////////

bool SRE::ResolutionList::IsValidWidth(int width) const
{
	return width >= MinWidth && width <= MaxWidth;
}

////////////////////////////////////////////////////////////////////////////

bool SRE::ResolutionList::IsValidHeight(int height) const
{
	return height >= MinHeight && height <= MaxHeight;
}

////////////////////////////////////////////////////////////////////////////

bool SRE::ResolutionList::Add(const Resolution &resolution)
{
	if (!IsValidWidth(resolution.width) || !IsValidHeight(resolution.height))
		return false;

	auto iterator = std::lower_bound(resolutionList.begin(), resolutionList.end(), resolution, std::greater<Resolution>());

	if (iterator != resolutionList.end() && *iterator == resolution)
	{
		if (iterator->added)
			return false;

		iterator->added = true;
		return true;
	}

	resolutionList.insert(iterator, Resolution(resolution.width, resolution.height));
	return true;
}

////////////////////////////////////////////////////////////////////////////

bool SRE::ResolutionList::IsAmdDefault(const Resolution &resolution) const
{
	auto iterator = std::find(AmdDefaultResolutionList.begin(), AmdDefaultResolutionList.end(), resolution);
	return iterator != AmdDefaultResolutionList.end() && !iterator->added;
}

////////////////////////////////////////////////////////////////////////////

bool SRE::ResolutionList::IsDefault(const Resolution &resolution) const
{
	switch (type)
	{
		case GpuType::Amd:
			return IsAmdDefault(resolution);

		default:
			return false;
	}
}

////////////////////////////////////////////////////////////////////////////

bool SRE::ResolutionList::AddDefault(const Resolution &resolution)
{
	if (!IsValidWidth(resolution.width) || !IsValidHeight(resolution.height))
		return false;

	if (!IsDefault(resolution))
		return Add(resolution);

	auto iterator = std::lower_bound(resolutionList.begin(), resolutionList.end(), resolution, std::greater<Resolution>());

	if (iterator != resolutionList.end() && *iterator == resolution)
		return false;

	resolutionList.insert(iterator, Resolution(resolution.width, resolution.height, false));
	return true;
}

////////////////////////////////////////////////////////////////////////////

bool SRE::ResolutionList::Get(int index, Resolution &resolution) const
{
	if (!IsValidIndex(index))
		return false;

	resolution = resolutionList[index];
	return true;
}

////////////////////////////////////////////////////////////////////////////

bool SRE::ResolutionList::Delete(int index)
{
	if (!IsValidIndex(index))
		return false;

	resolutionList.erase(resolutionList.begin() + index);
	return true;
}

////////////////////////////////////////////////////////////////////////////

bool SRE::ResolutionList::Clear()
{
	if (GetCount() <= 0)
		return false;

	resolutionList.clear();
	return true;
}

////////////////////////////////////////////////////////////////////////////

int SRE::ResolutionList::GetIndex(const Resolution &resolution) const
{
	auto iterator = std::find(resolutionList.begin(), resolutionList.end(), resolution);

	if (iterator == resolutionList.end())
		return -1;

	return static_cast<int>(iterator - resolutionList.begin());
}

////////////////////////////////////////////////////////////////////////////

bool SRE::ResolutionList::Exists(const Resolution &resolution) const
{
	return std::find(resolutionList.begin(), resolutionList.end(), resolution) != resolutionList.end();
}

////////////////////////////////////////////////////////////////////////////

bool SRE::ResolutionList::IsAdded(const Resolution &resolution) const
{
	auto iterator = std::find(resolutionList.begin(), resolutionList.end(), resolution);
	return iterator != resolutionList.end() && iterator->added;
}

////////////////////////////////////////////////////////////////////////////

std::basic_string<unsigned char> SRE::ResolutionList::GetAmdModes() const
{
	std::basic_string<unsigned char> value;

	for (const Resolution &resolution : resolutionList)
	{
		if (resolution.added)
			value += resolution.GetAmdMode();
	}

	return value;
}

////////////////////////////////////////////////////////////////////////////

std::basic_string<unsigned char> SRE::ResolutionList::GetAmdRestrictedModes() const
{
	std::basic_string<unsigned char> value;

	for (const Resolution &resolution : AmdDefaultResolutionList)
	{
		if (!Exists(resolution))
			value += resolution.GetAmdMode();
	}

	return value;
}

////////////////////////////////////////////////////////////////////////////

bool SRE::ResolutionList::SetAmdModes(const std::basic_string<unsigned char> &value)
{
	type = GpuType::Amd;
	resolutionList = AmdDefaultResolutionList;

	for (unsigned index = 0; index < value.size(); index += 8)
	{
		Resolution resolution;
		resolution.SetAmdMode(value.substr(index, 8));
		Add(resolution);
	}

	return true;
}

////////////////////////////////////////////////////////////////////////////

bool SRE::ResolutionList::DeleteAmdModes(const std::basic_string<unsigned char> &value)
{
	for (unsigned index = 0; index < value.size(); index += 8)
	{
		Resolution resolution;
		resolution.SetAmdMode(value.substr(index, 8));
		Delete(GetIndex(resolution));
	}

	return true;
}

////////////////////////////////////////////////////////////////////////////

std::wstring SRE::ResolutionList::GetNvidiaModes() const
{
	std::wstring string = L"{*}SHV";

	if (GetCount() <= 0)
	{
		string += L" 99999x99999";
	}
	else
	{
		for (const Resolution &resolution : resolutionList)
			string += L' ' + resolution.GetText();
	}

	string += L"=1" + L'\0';
	return string;
}

////////////////////////////////////////////////////////////////////////////

bool SRE::ResolutionList::SetNvidiaModes(const std::wstring &value)
{
	type = GpuType::Nvidia;

	if (value.find(L'=') == std::string::npos)
	{
		resolutionList = NvidiaDefaultResolutionList;
		return true;
	}

	resolutionList.clear();
	std::wstring string = value;
	std::replace(string.begin(), string.end(), L'\0', L' ');
	std::wstringstream stream(string);

	while (stream)
	{
		Resolution resolution;
		wchar_t x;

		stream >> resolution.width >> std::noskipws >> x >> resolution.height >> std::skipws;

		if (stream)
		{
			if (x == L'x')
				Add(resolution);

			stream.ignore(value.size(), L' ');
		}
		else if (!stream.eof())
		{
			stream.clear();
			stream.ignore(value.size(), L' ');
		}
	}

	return true;
}

////////////////////////////////////////////////////////////////////////////

template <class T>
bool SRE::ResolutionList::ReadStream(T &stream)
{
	bool valid = false;

	while (stream)
	{
		Resolution resolution;
		stream >> resolution.width;

		if (!stream)
		{
			if (!valid && stream.eof())
			{
				resolutionList.clear();
				valid = true;
			}

			break;
		}

		stream.get();
		stream >> resolution.height;

		if (!stream)
			break;

		if (!valid)
		{
			resolutionList.clear();
			valid = true;
		}

		int status = stream.get();

		if (stream && status == '*')
		{
			AddDefault(resolution);
			stream.get();
		}
		else
		{
			Add(resolution);
		}
	}

	return valid;
}

////////////////////////////////////////////////////////////////////////////

template <class T>
bool SRE::ResolutionList::WriteStream(T &stream) const
{
	for (const Resolution &resolution : resolutionList)
	{
		stream << resolution.width << 'x' << resolution.height;

		if (!resolution.added)
			stream << '*';

		stream << "\r\n";
	}

	return stream.good();
}

////////////////////////////////////////////////////////////////////////////

std::wstring SRE::ResolutionList::GetText() const
{
	std::wstringstream stream;
	WriteStream(stream);
	return stream.str();
}

////////////////////////////////////////////////////////////////////////////

bool SRE::ResolutionList::SetText(const std::wstring &text)
{
	std::wstringstream stream(text);
	return ReadStream(stream);
}

////////////////////////////////////////////////////////////////////////////

bool SRE::ResolutionList::Import(const std::wstring &file)
{
	std::ifstream stream(file);

	if (!stream)
		return false;

	return ReadStream(stream);
}

////////////////////////////////////////////////////////////////////////////

bool SRE::ResolutionList::Export(const std::wstring &file) const
{
	std::ofstream stream(file, std::ios::binary);

	if (!stream)
		return false;

	if (!WriteStream(stream))
		return false;

	stream.close();
	return stream.good();
}

////////////////////////////////////////////////////////////////////////////
